from .environment import Environment
from .core import Task, TaskState, Job, JobState, Workflow, WorkflowState
from . import parameters
from . import funcs
from . import exceptions


__all__ = [
    '__version__',
    'environment',
    'Task',
    'TaskState',
    'Job',
    'JobState',
    'Workflow',
    'WorkflowState',
    'parameters',
    'funcs',
    'exceptions',
]


__version__ = '0.1.4'


environment = Environment()


from __future__ import annotations
from typing import Iterable, Type
import os


def _listify_validator(scalar_validator, *, n=None, doc=None):

    def f(s):
        if isinstance(s, str):
            val = [scalar_validator(v.strip()) for v in s.split(',')
                    if v.strip()]
        # Allow any ordered sequence type -- generators, np.ndarray, pd.Series
        # -- but not sets, whose iteration order is non-deterministic.
        elif isinstance(s, Iterable) and not isinstance(s, (set, frozenset)):
            val = [scalar_validator(v) for v in s]
        else:
            raise ValueError(
                f"Expected str or other non-set iterable, but got {s}")
        if n is not None and len(val) != n:
            raise ValueError(
                f"Expected {n} values, but there are {len(val)} values in {s}")
        return val

    try:
        f.__name__ = "{}list".format(scalar_validator.__name__)
    except AttributeError:  # class instance.
        f.__name__ = "{}List".format(type(scalar_validator).__name__)
    f.__qualname__ = f.__qualname__.rsplit(".", 1)[0] + "." + f.__name__
    f.__doc__ = doc if doc is not None else scalar_validator.__doc__
    return f


def validate_any(s):
    return s


validate_anylist = _listify_validator(validate_any)


def _make_type_validator(cls: Type, *, allow_none=False) -> function:
    """
    Return a validator that converts inputs to *cls* or raises (and possibly
    allows ``None`` as well).
    """
    def validator(s):
        if allow_none and s is None:
            return None
        if cls is str and not isinstance(s, str):
            raise TypeError(f"{s} is not type str")
        if cls is int:
            try:
                val = round(s)
                if val != s:
                    raise ValueError(f"Could not convert {s!r} to int without precision loss")
            except TypeError:
                pass
        try:
            return cls(s)
        except (TypeError, ValueError) as e:
            raise ValueError(
                f'Could not convert {s!r} to {cls.__name__}') from e

    validator.__name__ = f"validate_{cls.__name__}"
    if allow_none:
        validator.__name__ += "_or_None"
    validator.__qualname__ = (
        validator.__qualname__.rsplit(".", 1)[0] + "." + validator.__name__)
    return validator


validate_str = _make_type_validator(str)


validate_str_or_None = _make_type_validator(str, allow_none=True)


validate_strlist = _listify_validator(
    validate_str, doc='return a list of strings')


validate_int = _make_type_validator(int)


validate_int_or_None = _make_type_validator(int, allow_none=True)


validate_intlist = _listify_validator(
    validate_int, doc='return a list of ints')


validate_float = _make_type_validator(float)


validate_float_or_None = _make_type_validator(float, allow_none=True)


validate_floatlist = _listify_validator(
    validate_float, doc='return a list of floats')


def validate_bool(s):
    if s not in (True, False):
        raise ValueError(f"Could not convert {s!r} to bool")


def _make_range_validator(*, _min: int | float = float('-inf'), _max: int | float = float('inf')):

    def validate_range(s):
        if not _min <= s <= _max:
            raise ValueError(f"{s!r} is not in range from {_min!r} to {_max!r}")
        return s

    return validate_range


def _make_options_validator(*, options: Iterable = []):

    def validate_options(s):
        if s not in options:
            raise ValueError(f"{s!r} is not a valid option")
        return s

    return validate_options


def _validate_pathlike(s):
    if isinstance(s, (str, os.PathLike)):
        # Store value as str because savefig.directory needs to distinguish
        # between "" (cwd) and "." (cwd, but gets updated by user selections).
        return os.fsdecode(s)
    else:
        raise TypeError(f"Cound not convert {s!r} to path str")




from __future__ import annotations
from abc import ABC, abstractmethod
from typing import Any, Optional

from . import validators


__all__ = [
    "Parameter",
    "AnyParameter",
    "IntParameter",
    "FloatParameter",
    "StrParameter",
    "BoolParameter",
    "ListParameter",
    "IntListParameter",
    "FloatListParameter",
    "StrListParameter",
    "OptionsParameter",
]


class Parameter(ABC):

    __count = 0

    def __init__(self, *, display_name: Optional[str] = None):
        self.__class__.__count += 1
        self.__storage_name = f"_{self.__class__.__name__}#{self.__count}"
        self.__display_name = display_name
    
    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(display_name={self.display_name!r})"

    def __set__(self, instance, value):
        value = self.validate(instance, value)
        setattr(instance, self.__storage_name, value)
        
    def __get__(self, instance, owner):
        if instance is None:
            return self
        else:
            try:
                return getattr(instance, self.__storage_name)
            except AttributeError as e:
                raise AttributeError(f"This {self.__class__.__name__} of {instance.__class__.__name__} is not assigned yet.") from e

    @property
    def display_name(self):
        return self.__display_name

    @abstractmethod
    def validate(self, instance, value) -> Any:
        """Validate and returns the converted value."""


class AnyParameter(Parameter):

    def validate(self, instance, value) -> Any:
        return validators.validate_any(value)


class IntParameter(Parameter):
    
    def __init__(self, min: int | float = float('-inf'), max: int | float = float('inf'), allow_none: bool = False, *, display_name: Optional[str] = None) -> None:
        super().__init__(display_name=display_name)
        self._min = min
        self._max = max
        self._allow_none = allow_none

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(min={self._min!r}, max={self._max}, allow_none={self._allow_none!r}, display_name={self.display_name!r})"

    def validate(self, instance, value) -> Optional[int]:
        value = validators.validate_int_or_None(value) if self._allow_none else validators.validate_int(value)
        if value is not None:
            value = validators._make_range_validator(_min=self._min, _max=self._max)(value)
        return value


class FloatParameter(Parameter):

    def __init__(self, min: int | float = float('-inf'), max: int | float = float('inf'), allow_none: bool = False, *, display_name: Optional[str] = None) -> None:
        super().__init__(display_name=display_name)
        self._min = min
        self._max = max
        self._allow_none = allow_none

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(min={self._min!r}, max={self._max!r}, allow_none={self._allow_none!r}, display_name={self.display_name!r})"

    def validate(self, instance, value) -> Optional[float]:
        value = validators.validate_float_or_None(value) if self._allow_none else validators.validate_float(value)
        if value is not None:
            value = validators._make_range_validator(_min=self._min, _max=self._max)(value)
        return value


class StrParameter(Parameter):

    def __init__(self, allow_none: bool = False, *, display_name: Optional[str] = None) -> None:
        super().__init__(display_name=display_name)
        self._allow_none = allow_none

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(allow_none={self._allow_none!r}, display_name={self.display_name!r})"
    
    def validate(self, instance, value) -> Optional[str]:
        value = validators.validate_str_or_None(value) if self._allow_none else validators.validate_str(value)
        return value


class BoolParameter(Parameter):

    def validate(self, instance, value):
        return validators.validate_bool(value)


class ListParameter(Parameter):

    def __init__(self, n: Optional[int] = None, *, display_name: Optional[str] = None):
        super().__init__(display_name=display_name)
        self._n = n

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(n={self._n!r}, display_name={self.display_name!r})"
    
    def validate(self, instance, value):
        c_val = validators.validate_anylist(value)
        if self._n is not None:
            if len(c_val) != self._n:
                raise ValueError(f"Expected {self._n} values, but there are {len(c_val)} values in {value}")
        return c_val


class IntListParameter(Parameter):

    def __init__(self, n: Optional[int] = None, *, display_name: Optional[str] = None):
        super().__init__(display_name=display_name)
        self._n = n

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(n={self._n!r}, display_name={self.display_name!r})"

    def validate(self, instance, value):
        c_val = validators.validate_intlist(value)
        if self._n is not None:
            if len(c_val) != self._n:
                raise ValueError(f"Expected {self._n} values, but there are {len(c_val)} values in {value}")
        return c_val


class FloatListParameter(Parameter):

    def __init__(self, n: Optional[int] = None, *, display_name: Optional[str] = None):
        super().__init__(display_name=display_name)
        self._n = n

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(n={self._n!r}, display_name={self.display_name!r})"

    def validate(self, instance, value):
        c_val = validators.validate_floatlist(value)
        if self._n is not None:
            if len(c_val) != self._n:
                raise ValueError(f"Expected {self._n} values, but there are {len(c_val)} values in {value}")
        return c_val


class StrListParameter(Parameter):

    def __init__(self, n: Optional[int] = None, *, display_name: Optional[str] = None):
        super().__init__(display_name=display_name)
        self._n = n

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(n={self._n!r}, display_name={self.display_name!r})"

    def validate(self, instance, value):
        c_val = validators.validate_strlist(value)
        if self._n is not None:
            if len(c_val) != self._n:
                raise ValueError(f"Expected {self._n} values, but there are {len(c_val)} values in {value}")
        return c_val


class OptionsParameter(Parameter):

    def __init__(self, options=[], *, display_name: Optional[str] = None):
        super().__init__(display_name=display_name)
        self._options = list(options)

    def validate(self, instance, value):
        return validators._make_options_validator(options=self._options)(value)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lightningflow']

package_data = \
{'': ['*']}

install_requires = \
['networkx>=2.8,<3.0']

setup_kwargs = {
    'name': 'lightningflow',
    'version': '0.1.4',
    'description': 'A workflow template designed for optical transceiver test.',
    'long_description': None,
    'author': 'Chongjun Lei',
    'author_email': 'chongjun.lei@neophotonics.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/asakiasako/LIGHTNINGFLOW',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Optional
from dataclasses import dataclass
from io import TextIOWrapper
from sys import stdout


__all__ = ['Environment']


@dataclass(init=False)
class Environment:
    """A singoten class to store environments globally."""
    __instance = None
    def __new__(cls: type):
        cls.__instance = cls.__instance or object.__new__(cls)
        return cls.__instance
    outputTarget: TextIOWrapper = stdout
    currentTask: Optional[str] = None
    currentJob: Optional[str] = None
    currentWorkflow: Optional[str] = None
    currentWorkflowInstance: Optional['Workflow'] = None
